@@ -51,6 +51,7 @@
 	reasonable value (30, before it defaulted to 0, which made dashlines
 	look very sparse).
   3. Arc boxes are now implemented.
+  4. New arrow types implemented
 
   Known problems:
 	In the fig2dev documentation you should add a recommendation not to use
@@ -127,6 +128,7 @@ static void genepic_open_spline(), genepic_closed_spline();
 static quadratic_spline();
 static bezier_spline();
 static arc_tangent();
+static arc_arrow();
 static draw_arrow_head();
 
 /* Structure for Point with "double" values */
@@ -217,6 +219,7 @@ char *Preamble="\\documentstyle[epic,eepic]{article}\n\\begin{document}\n\\begin
 char *Postamble="\\end{center}\n\\end{document}\n";
 int VarWidth=FALSE;
 int DashStretch=30;
+double ArrowScale=1.0;
 
 void genepic_option(opt, optarg)
 char opt, *optarg;
@@ -224,6 +227,10 @@ char opt, *optarg;
   	int loop, i;
 
         switch (opt) {
+	case 'A':
+	  ArrowScale = atof(optarg);
+	  break;
+
 	case 'a':
 	    fprintf(stderr, "warning: genepic option -a obsolete");
 	    break;
@@ -421,14 +428,13 @@ int w;
     int old_width;
 
     if (w < 0) return;
-    w/= DashScale; /* Dashscale is also computed in 80th of an inch */
     old_width=CurWidth;
     CurWidth = (w >= LineThick) ? (VarWidth ? w : ThickLines) : ThinLines;
     if (old_width != CurWidth) {
 	if (CurWidth==ThinLines) {
 	    fprintf(tfp, "\\thinlines\n");
 	} else if (VarWidth) {
-	    fprintf(tfp, "\\allinethickness{%d}%%\n",w);
+	    fprintf(tfp, "\\allinethickness{%4.3fpt}%%\n",w*80/DPI);
 	} else {
 	    fprintf(tfp, "\\thicklines\n");
 	}
@@ -659,7 +665,7 @@ int type, color;
 	    if (--loop1 > 0)
 		fprintf(tfp, "\n\t");
 	    else
-		fprintf(tfp, "}%\n");
+		fprintf(tfp, "}\n");
 	}
     }
 }
@@ -682,6 +688,13 @@ F_line *line;
 	fprintf(tfp, "\\drawline(%d,%d)(%d,%d)\n", p->x, p->y, p->x, p->y);
 	return;
     }
+    convertCS(q);
+    if (line->back_arrow) {
+	draw_arrow_head(q, p, line->back_arrow->ht, line->back_arrow->wid,
+			line->back_arrow->type, line->back_arrow->style, 
+			line->back_arrow->thickness);
+    	if (Verbose) fprintf(tfp, "%%\n");
+    }
     if (line->type == T_ARC_BOX) { /* A box with rounded corners */
 
       if (TeXLang == Epic || /* Sorry, can't do more */
@@ -697,7 +710,6 @@ F_line *line;
 	llx = urx = p->x;
 	lly = ury = p->y;
 	while (q != NULL) {
-	  convertCS(q);
 	  if (q->x < llx) {
 	    llx = q->x;
 	  } else if (q->x > urx) {
@@ -708,7 +720,8 @@ F_line *line;
 	  } else if (q->y > ury) {
 	    ury = q->y;
 	  }
-	  q = q->next;
+	  if ((q = q->next))
+	     convertCS(q);
 	}
 	r= line->radius;
 	fprintf(tfp, "\\put(%d,%d){\\arc{%d}{1.5708}{3.1416}}\n", llx+r, lly+r, 2*r);
@@ -742,7 +755,6 @@ F_line *line;
 	    llx = urx = p->x;
 	    lly = ury = p->y;
 	    while (q != NULL) {
-	        convertCS(q);
 	        if (q->x < llx) {
 	            llx = q->x;
 	        } else if (q->x > urx) {
@@ -753,7 +765,8 @@ F_line *line;
 	        } else if (q->y > ury) {
 	            ury = q->y;
 	        }
-	        q = q->next;
+	        if ((q = q->next))
+		   convertCS(q);
 	    }
 	    switch(LineStyle) {
 	    case SOLID_LINE:
@@ -776,7 +789,6 @@ F_line *line;
       fprintf(tfp, "%%\n%% A polyline\n%%\n");
     }
     set_pattern(line->fill_style, line->fill_color);
-    convertCS(q);
     switch (LineStyle) {
     case SOLID_LINE:
 	if (q->next != NULL && strcmp(LnCmd,"path")==0) {
@@ -838,12 +850,6 @@ F_line *line;
 	fprintf(tfp, "(%d,%d)\n", q->x, q->y);
     }
 #endif
-    if (line->back_arrow) {
-	draw_arrow_head(q, p, line->back_arrow->ht, line->back_arrow->wid,
-			line->back_arrow->type, line->back_arrow->style, 
-			line->back_arrow->thickness);
-    	if (Verbose) fprintf(tfp, "%%\n");
-    }
     if (line->for_arrow) {
 	draw_arrow_head(p, q, line->for_arrow->ht, line->for_arrow->wid,
 			line->for_arrow->type,line->for_arrow->style,
@@ -854,21 +860,21 @@ F_line *line;
 
 set_style(style, dash_len)
 int style;
-float dash_len;
+double dash_len;
 {
     LineStyle = style;
     if (LineStyle == DASH_LINE) {
         switch (DashType) {
         case DottedDash:
             LineStyle = DOTTED_LINE;
-	    DotDist = dash_len * DashScale;
+	    DotDist = round(dash_len * DashScale);
             break;
         default:
-            DashLen = dash_len * DashScale;
+            DashLen = round(dash_len * DashScale);
             break;
         }
     } else if (LineStyle == DOTTED_LINE) {
-	DotDist = dash_len * DashScale;
+	DotDist = round(dash_len * DashScale);
     }
 
 }
@@ -908,6 +914,12 @@ F_spline *spl;
     q = p->next;
     convertCS(p);
     convertCS(q);
+    if (spl->back_arrow) {
+      draw_arrow_head(q, p, spl->back_arrow->ht, spl->back_arrow->wid,
+                      spl->back_arrow->type,spl->back_arrow->style,
+                      spl->back_arrow->thickness);
+      if (Verbose) fprintf(tfp, "%%\n");
+    }
     if (q->next == NULL) {
 	fprintf(tfp, "\\%s(%d,%d)(%d,%d)\n", LnCmd,
 	       p->x, p->y, q->x, q->y);
@@ -949,12 +961,6 @@ F_spline *spl;
         q=r;
 	fprintf(tfp, "\n");
     }
-    if (spl->back_arrow) {
-	draw_arrow_head(q, p, spl->back_arrow->ht, spl->back_arrow->wid,
-			spl->back_arrow->type,spl->back_arrow->style,
-			spl->back_arrow->thickness);
-    	if (Verbose) fprintf(tfp, "%%\n");
-    }
     if (spl->for_arrow) {
 	draw_arrow_head(p, q, spl->for_arrow->ht, spl->for_arrow->wid,
 			spl->for_arrow->type, spl->for_arrow->style,
@@ -1061,6 +1067,15 @@ F_spline *spl;
     pt1r.y = cp1->ry;
     fconvertCS(&pt1l);
     fconvertCS(&pt1r);
+    if (spl->back_arrow) {
+      tmpfpt.x = p1->x;
+      tmpfpt.y = p1->y;
+      fdraw_arrow_head(&pt1r, &tmpfpt,
+                       spl->back_arrow->ht, spl->back_arrow->wid,
+                       spl->back_arrow->type, spl->back_arrow->style,
+                       spl->back_arrow->thickness);
+      if (Verbose) fprintf(tfp, "%%\n");
+    }
 
     for (p2 = p1->next, cp2 = cp1->next; p2 != NULL;
 	 p1 = p2, pt1r = pt2r, p2 = p2->next, cp2 = cp2->next) {
@@ -1079,16 +1094,6 @@ F_spline *spl;
 	fprintf(tfp, "\n");
     }
 
-    if (spl->back_arrow) {
-	tmpfpt.x = p1->x;
-	tmpfpt.y = p1->y;
-	fdraw_arrow_head(&pt1r, &tmpfpt, 
-			 spl->back_arrow->ht, spl->back_arrow->wid,
-			 spl->back_arrow->type, spl->back_arrow->style,
-			 spl->back_arrow->thickness);
-    	if (Verbose) fprintf(tfp, "%%\n");
-    }
-
     if (spl->for_arrow) {
 	tmpfpt.x = p1->x;
 	tmpfpt.y = p1->y;
@@ -1384,7 +1389,7 @@ F_arc *arc;
         }
     }
     if (arc->for_arrow) {
-	arc_tangent(&ctr, &pt2, arc->direction, &tmp);
+	arc_arrow(&ctr, &pt2, arc->direction, arc->for_arrow, &tmp);
 	fdraw_arrow_head(&tmp, &pt2,
 			 arc->for_arrow->ht*arrowfactor,
 			 arc->for_arrow->wid*arrowfactor,
@@ -1394,7 +1399,7 @@ F_arc *arc;
     	if (Verbose) fprintf(tfp, "%%\n");
     }
     if (arc->back_arrow) {
-	arc_tangent(&ctr, &pt1, !arc->direction, &tmp);
+	arc_arrow(&ctr, &pt1, !arc->direction, arc->back_arrow, &tmp);
 	fdraw_arrow_head(&tmp, &pt1,
 			 arc->back_arrow->ht*arrowfactor,
 			 arc->back_arrow->wid*arrowfactor,
@@ -1441,6 +1446,60 @@ int direction;
     }
 }
 
+/*************************** ARROWS *****************************
+
+ arc_arrow - Computes a point on a line which is a chord to the 
+        arc specified by center pt1 and endpoint pt2, where the 
+        chord intersects the arc arrow->ht from the endpoint.
+
+ May give strange values if the arrow.ht is larger than about 1/4 of
+ the circumference of a circle on which the arc lies.
+
+  This function is copied from xfig-3.2.0-beta3/u_draw.c 
+       compute_arcarrow_angle. Thanks to the author.
+
+****************************************************************/
+
+static arc_arrow(pt1, pt2, direction, arrow, pt3)
+FPoint *pt1, *pt2, *pt3;
+int direction;
+F_arrow *arrow;
+{
+    double	r, alpha, beta, dy, dx, x1, x2, y1, y2;
+    double	lpt,h;
+
+    x1=pt1->x;
+    y1=pt1->y;
+
+    x2=pt2->x;
+    y2=pt2->y;
+
+    dy=y2-y1;
+    dx=x2-x1;
+    r=sqrt(dx*dx+dy*dy);
+    h = arrow->ht;
+    /* lpt is the amount the arrowhead extends beyond the end of the line */
+    lpt = arrow->thickness/30.0/(arrow->wid/h/2.0);
+    /* add this to the length */
+    h += lpt;
+
+    /* radius too small for this method, use normal method */
+    if (h > 2.0*r) {
+	arc_tangent(pt1,pt2,direction,pt3);
+	return;
+    }
+
+    beta=atan2(dx,dy);
+    if (direction) {
+	alpha=2*asin(h/2.0/r);
+    } else {
+	alpha=-2*asin(h/2.0/r);
+    }
+
+    pt3->x=round(x1+r*sin(beta+alpha));
+    pt3->y=y1+r*cos(beta+alpha);
+}
+
 rtop(x, y, r, th)
 double x, y, *r, *th;
 {
@@ -1473,6 +1532,9 @@ double arrowht, arrowwid, thickness;
     double x,y, xb,yb,dx,dy,l,sina,cosa;
     double xc, yc, xd, yd;
 
+    arrowht/= ArrowScale;
+    arrowwid/= ArrowScale;
+
     x1 = pt1->x;
     y1 = pt1->y;
     x2 = pt2->x;
