@@ -83,12 +83,6 @@ char opt, *optarg;
 		break;
 
 	case 's':
-		if (font_size <= 0 || font_size > MAXFONTSIZE) {
-			fprintf(stderr,
-				"warning: font size %d out of bounds\n", font_size);
-		}
-		break;
-
 	case 'm':
 	case 'L':
 		break;
@@ -142,7 +136,8 @@ F_compound	*objects;
 	coord_system = objects->nwcorner.y;
 	if (coord_system == 2) CONV = 1;
 
-	fprintf(tfp, ".PS\n.ps %d\n", font_size);	/* PIC preamble */
+	fprintf(tfp, ".PS\n.ps %d\n", 		/* PIC preamble */
+		font_size?font_size:DEFAULT_FONT_SIZE);
 }
 
 void
@@ -166,7 +161,7 @@ int	w;
 {
 	static int	cur_thickness = -1;
 
-	LineThickness = w;
+	LineThickness = w*80/ppi;
 
 	/*
 	if (w == 0) return;
@@ -187,7 +182,7 @@ AddThickness()
 static void
 set_style(s, v)
 int	s;
-float	v;
+double	v;
 {
 	static float	style_val = -1;
 
@@ -195,7 +190,7 @@ float	v;
 	    if (v == style_val) return;
 	    if (v == 0.0) return;
 	    style_val = v;
-	    fprintf(tfp, "dashwid = %.3fi\n", style_val/ppi);
+	    fprintf(tfp, "dashwid = %.3f\n", (float)(style_val/80.0));
 	    }
 }
 
@@ -240,8 +235,8 @@ F_line *l;
     if (l->thickness == 0)
       fprintf(tfp, " invis");
     else
-      if (l->style_val > 0.0)
-      { if (l->style == DASH_LINE)
+      if (l->style_val > 0.0) {
+	if (l->style == DASH_LINE)
 	  fprintf(tfp, " dashed");
 	else if (l->style == DOTTED_LINE)
 	  fprintf(tfp, " dotted");
@@ -496,6 +491,13 @@ F_arc	*a;
 
 	fprintf(tfp, "arc ");
 
+	if (a->style_val > 0.0) {
+	    if (a->style == DASH_LINE)
+		fprintf(tfp, " dashed");
+	    else if (a->style == DOTTED_LINE)
+		fprintf(tfp, " dotted");
+	}
+
 	/*rja: Attach arrowhead as required */
 	if ((a->for_arrow) && (a->back_arrow))
 	  fprintf(tfp, " <->");
