@@ -47,7 +47,11 @@
 #include <math.h>
 #include <pwd.h>
 #include <errno.h>
+#ifndef __NetBSD__
+#if (! (defined(BSD) && (BSD >= 199306)))
 extern char *sys_errlist[];
+#endif
+#endif
 #include "pi.h"
 #include "fig2dev.h"
 #include "object.h"
@@ -57,10 +61,16 @@ extern char *sys_errlist[];
 
 /* for the xpm package */
 #ifdef USE_XPM
-#include <X11/xpm.h>
+#include <xpm.h>
 int	XpmReadFileToXpmImage();
 #endif
 
+#ifdef I18N
+#include <stdlib.h>
+extern Boolean support_i18n;  /* enable i18n support? */
+static Boolean enable_composite_font = FALSE;
+#endif /* I18N */
+
 /* for the version nubmer */
 #include "../../patchlevel.h"
 
@@ -69,38 +79,11 @@ int	XpmReadFileToXpmImage();
 
 int	ReadFromBitmapFile();
 
-/* ratio of resolution to 80ppi */
-extern float	THICK_SCALE;
-
 typedef struct _point
 {
     int x,y;
 } Point;
 
-struct pagedef
-{
-    char *name;			/* name for page size */
-    int width;			/* page width in points */
-    int height;			/* page height in points */
-};
-
-struct pagedef pagedef[] =
-{
-    {"A4", 595, 842}, 		/* 21cm x 29.7cm */
-    {"B5", 516, 729}, 		/* 18.2cm x 25.7cm */
-    {"Letter", 612, 792}, 	/* 8.5" x 11" */
-    {"Legal", 612, 1008}, 	/* 8.5" x 14" */
-    {"Ledger", 1224, 792}, 	/*  17" x 11" */
-    {"Tabloid", 792, 1224}, 	/*  11" x 17" */
-    {NULL, 0, 0}
-};
-
-#ifdef A4
-char *pagesize = "A4";
-#else
-char *pagesize = "Letter";
-#endif
-
 #define		POINT_PER_INCH		72
 #define		ULIMIT_FONT_SIZE	300
 
@@ -112,13 +95,12 @@ int		xoff=0;
 int		yoff=0;
 static int	coord_system;
 static int	resolution;
-int		show_page = 0;
+Boolean		show_page = FALSE;
 static double	cur_thickness = 0.0;
 static int	cur_joinstyle = 0;
 static int	cur_capstyle = 0;
 int		pages;
 int		no_obj = 0;
-int		multi_page = FALSE;
 
 /* arrowhead arrays */
 Point		bpoints[5], fpoints[5];
@@ -136,7 +118,7 @@ static		draw_arrow();
 static		iso_text_exist();
 static		encode_all_fonts();
 static		ellipse_exist();
-static		normal_spline_exist();
+static		approx_spline_exist();
 
 #define SHADEVAL(F)	1.0*(F)/(NUMSHADES-1)
 #define TINTVAL(F)	1.0*(F-NUMSHADES+1)/NUMTINTS
@@ -238,23 +220,35 @@ char *optarg;
 	    	break;
 
 	case 'c':			/* center figure */
-	    	center = 1;
+	    	center = TRUE;
+		centerspec = TRUE;	/* user-specified */
 		break;
 
 	case 'e':			/* don't center ('e' means edge) figure */
-	    	center = 0;
+	    	center = FALSE;
+		centerspec = TRUE;	/* user-specified */
 		break;
 
 	case 'M':			/* multi-page option */
-		multi_page = 1;
+		multi_page = TRUE;
+		multispec = TRUE;	/* user has overridden anything in file */
+		break;
+
+	case 'S':			/* turn off multi-page option */
+		multi_page = FALSE;
+		multispec = TRUE;	/* user has overridden anything in file */
 		break;
 
 	case 'P':			/* add showpage */
-		show_page = 1;
+		show_page = TRUE;
 		break;
 
+      	case 'm':			/* magnification (already parsed in main) */
+		magspec = TRUE;		/* user-specified */
+		break;
+
+	/* don't do anything for language and font size (already parsed in main) */
       	case 'L':			/* language */
-      	case 'm':			/* magnification */
 	case 's':			/* default font size */
 		break;
 
@@ -263,13 +257,13 @@ char *optarg;
 		break;
 
       	case 'l':			/* landscape mode */
-		landscape = 1;		/* override the figure file setting */
-		orientspec = 1;		/* user-specified */
+		landscape = TRUE;	/* override the figure file setting */
+		orientspec = TRUE;	/* user-specified */
 		break;
 
       	case 'p':			/* portrait mode */
-		landscape = 0;		/* override the figure file setting */
-		orientspec = 1;		/* user-specified */
+		landscape = FALSE;	/* override the figure file setting */
+		orientspec = TRUE;	/* user-specified */
 		break;
 
 	case 'x':			/* x offset on page */
@@ -280,14 +274,9 @@ char *optarg;
 		yoff = atoi(optarg);
 		break;
 
-	case 'z':			/* pagesize */
-		if ((pagesize = (char *) malloc (strlen (optarg) + 1)) == NULL)
-		   {
-		   (void) fprintf (stderr, "No memory\n");
-		   exit (1);
-		   }
-
-		(void) strcpy (pagesize, optarg);
+	case 'z':			/* papersize */
+		(void) strcpy (papersize, optarg);
+		paperspec = TRUE;	/* user-specified */
 		break;
 
 	default:
@@ -305,54 +294,66 @@ F_compound	*objects;
 	struct passwd	*who;
 	time_t		when;
 	int		itmp;
-	struct pagedef	*pd;
+	int		clipx, clipy;
+	struct paperdef	*pd;
 
 	resolution = objects->nwcorner.x;
 	coord_system = objects->nwcorner.y;
 	scalex = scaley = mag * POINT_PER_INCH / (double)resolution;
+	/* if metric, adjust scale for difference between 
+	   FIG PIX/CM (450) and actual (472.44) */
+	if (metric) {
+		scalex *= 80.0/76.2;
+		scaley *= 80.0/76.2;
+	}
+
 	/* convert to point unit */
 	llx = (int)floor(llx * scalex); lly = (int)floor(lly * scaley);
+	/* save upper bounds before scaling for clipping later */
+	clipx = urx;
+	clipy = ury;
 	urx = (int)ceil(urx * scalex); ury = (int)ceil(ury * scaley);
 
-
-        for (pd = pagedef; pd -> name != NULL; pd++)
-	   if (strcmp (pagesize, pd -> name) == 0)
-	      {
-	      pagewidth = pd -> width;
-	      pageheight = pd -> height;
-	      }
+        for (pd = paperdef; pd->name != NULL; pd++)
+	    if (strcasecmp (papersize, pd->name) == 0) {
+		pagewidth = pd->width;
+		pageheight = pd->height;
+		strcpy(papersize,pd->name);	/* use the "nice" form */
+		break;
+	    }
 	
-	if (pagewidth < 0 || pageheight < 0)
-	   {
-	   (void) fprintf (stderr, "Unknown page size `%s'\n", pagesize);
-	   exit (1);
-	   }
+	if (pagewidth < 0 || pageheight < 0) {
+	    (void) fprintf (stderr, "Unknown paper size `%s'\n", papersize);
+	    exit (1);
+	}
 
 	if (landscape) {
-	   itmp = pageheight; pageheight = pagewidth; pagewidth = itmp;
-	   itmp = llx; llx = lly; lly = itmp;
-	   itmp = urx; urx = ury; ury = itmp;
+	    if (show_page) {
+		itmp = pageheight; pageheight = pagewidth; pagewidth = itmp;
+		itmp = llx; llx = lly; lly = itmp;
+		itmp = urx; urx = ury; ury = itmp;
+	    }
 	}
 	if (show_page) {
-	   if (center) {
-	      if (landscape) {
-		 origx = (pageheight - urx - llx)/2.0;
-		 origy = (pagewidth - ury - lly)/2.0;
-	      } else {
-		 origx = (pagewidth - urx - llx)/2.0;
-		 origy = (pageheight + ury + lly)/2.0;
-	      }
-	   } else {
-	      origx = 0.0;
-	      origy = landscape ? 0.0 : pageheight;
+	    if (center) {
+		if (landscape) {
+		    origx = (pageheight - urx - llx)/2.0;
+		    origy = (pagewidth - ury - lly)/2.0;
+		} else {
+		    origx = (pagewidth - urx - llx)/2.0;
+		    origy = (pageheight + ury + lly)/2.0;
+		}
+	    } else {
+		origx = 0.0;
+		origy = landscape ? 0.0 : pageheight;
 	   }
 	} else {
-	   origx = -llx;
-	   origy = landscape ? -lly : ury;
+	    origx = -llx;
+	    origy = (landscape && show_page) ? -lly : ury;
 	}
 
 	/* finally, adjust by any offset the user wants */
-	if (landscape) {
+	if (landscape && show_page) {
 	    origx += yoff;
 	    origy += xoff;
 	} else {
@@ -377,19 +378,15 @@ F_compound	*objects;
 	   fprintf(tfp, "%%%%For: %s@%s (%s)\n",
 			who->pw_name, host, who->pw_gecos);
 
-	/* put in the magnification for information purposes */
-	/* This is not DSC so don't use two % */
-	fprintf(tfp, "%%Magnification: %.2f\n",mag);
-
 	if (!center) {
-	   if (landscape)
+	   if (landscape && show_page)
 		pages = (urx/pageheight+1)*(ury/pagewidth+1);
 	   else
 		pages = (urx/pagewidth+1)*(ury/pageheight+1);
 	} else {
 	   pages = 1;
 	}
-	if (landscape) {
+	if (landscape && show_page) {
 	   fprintf(tfp, "%%%%Orientation: Landscape\n");
 	   fprintf(tfp, "%%%%BoundingBox: %d %d %d %d\n",
 	      (int)origx+llx, (int)origy+lly, (int)origx+urx, (int)origy+ury);
@@ -400,10 +397,17 @@ F_compound	*objects;
 	}
 	fprintf(tfp, "%%%%Pages: %d\n", show_page ? pages : 0 );
 	fprintf(tfp, "%%%%BeginSetup\n");
-	fprintf(tfp, "%%%%IncludeFeature: *PageSize %s\n", pagesize);
+	/* only include a pagesize command if not EPS */
+	if (show_page) {
+	    fprintf(tfp, "%%%%IncludeFeature: *PageSize %s\n", papersize);
+	}
 	fprintf(tfp, "%%%%EndSetup\n");
 
+	/* put in the magnification for information purposes */
+	fprintf(tfp, "%%%%Magnification: %.4f\n",mag);
+
 	fprintf(tfp, "%%%%EndComments\n");
+
 	if (pats_used)
 		fprintf(tfp,"/MyAppDict 100 dict dup begin def\n");
 	fprintf(tfp, "%s", BEGIN_PROLOG1);
@@ -416,8 +420,8 @@ F_compound	*objects;
 	/* must specify translation/rotation before definition of fill patterns */
 	fprintf(tfp, "save\n");
 	fprintf(tfp, "%.1f %.1f translate\n", origx, origy);
-	/* also flip y if necessary */
-	if (landscape) {
+	/* also flip y if necessary, but *only* if to PostScript and not EPS */
+	if (landscape && show_page) {
 	    fprintf(tfp, " 90 rotate\n");
 	}
 	if (coord_system == 2) {
@@ -443,16 +447,42 @@ F_compound	*objects;
 	}
 	if (ellipse_exist(objects))
 		fprintf(tfp, "%s\n", ELLIPSE_PS);
-	if (normal_spline_exist(objects))
+	if (approx_spline_exist(objects))
 		fprintf(tfp, "%s\n", SPLINE_PS);
+#ifdef I18N
+	if (support_i18n && iso_text_exist(objects)) {
+	  char *libdir, *locale;
+	  char filename[512], str[512];
+	  FILE *fp;
+	  libdir = getenv("FIG2DEV_LIBDIR");
+	  if (libdir == NULL) libdir = FIG2DEV_LIBDIR;
+	  locale = getenv("LANG");
+	  if (locale == NULL) {
+	    fprintf(stderr, "fig2dev: LANG not defined; assuming C locale\n");
+	    locale = "C";
+	  }
+	  sprintf(filename, "%s/%s.ps", libdir, locale);
+	  /* get filename like ``/usr/local/lib/fig2dev/japanese.ps'' */
+	  fp = fopen(filename, "r");
+	  if (fp == NULL) {
+	    fprintf(stderr, "fig2dev: can't open file: %s\n", filename);
+	  } else {
+	    while (fgets(str, sizeof(str), fp)) {
+	      if (strstr(str, "CompositeRoman")) enable_composite_font = TRUE;
+	      fputs(str, tfp);
+	    }
+	    fclose(fp);
+	  }
+	}
+#endif /* I18N */
 	
 	fprintf(tfp, "%s\n", END_PROLOG);
 	fprintf(tfp, "$F2psBegin\n");
 	fprintf(tfp, "10 setmiterlimit\n");	/* make like X server (11 degrees) */
-	/* set initial clipping area to size of page (this is needed for
-	   later clipping by arrowheads */
+	/* set initial clipping area to size of the bounding box plus a margin
+	   (this is needed for later clipping by arrowheads */
 	fprintf(tfp, "n %d %d m %d %d l %d %d l %d %d l cp clip\n",
-			0,pageheight,0,0,pagewidth,0,pagewidth,pageheight);
+			-1000,clipy+1000,-1000,-1000,clipx+1000,-1000,clipx+1000,clipy+1000);
 
  	if ( multi_page ) {
 	    fprintf(tfp, "initmatrix\n");
@@ -472,14 +502,15 @@ genps_end()
 
     if (multi_page) {
        page = 1;
-       h = (landscape? pagewidth: pageheight);
-       w = (landscape? pageheight: pagewidth);
+       h = ((landscape && show_page)? pagewidth: pageheight);
+       w = ((landscape && show_page)? pageheight: pagewidth);
        for (dy=0; dy < (ury-h*0.1); dy += h*0.9) {
 	 for (dx=0; dx < (urx-w*0.1); dx += w*0.9) {
 	    fprintf(tfp, "%%%%Page: %d %d\n",page,page);
-	    fprintf(tfp,"%.1f %.1f tr",
-		-(origx+dx), (origy+(landscape?-dy:dy)));
-	    if (landscape) {
+	    fprintf(tfp, "gs\n");
+	    fprintf(tfp,"%.1f %.1f tr", 
+		-dx, ((landscape && show_page)? -dy:(dy+h*0.9)));
+	    if (landscape && show_page) {
 	       fprintf(tfp, " 90 rot");
 	    }
 	    if (coord_system == 2) {
@@ -488,21 +519,22 @@ genps_end()
 	    fprintf(tfp, " %.3f %.3f sc\n", scalex, scaley);
 	    for (i=0; i<no_obj; i++) {
 	       fprintf(tfp, "o%d ", i);
-	       if (!(i%20)) fprintf(tfp, "\n", i);
+	       if (!(i%20)) 
+		  fprintf(tfp, "\n");
 	    }
+	    fprintf(tfp, "gr\n");
 	    fprintf(tfp, "showpage\n");
 	    page++;
 	 }
        }
-    } else {
-	if (show_page) {
-	    fprintf(tfp, "showpage\n");
-	}
-    }
+    } 
     fprintf(tfp, "$F2psEnd\n");
     fprintf(tfp, "rs\n");
     if (pats_used)
 	fprintf(tfp, "end\n");		/* close off MyAppDict */
+    /* add showpage if requested */
+    if (!multi_page && show_page)
+	fprintf(tfp, "showpage\n");
 }
 
 static
@@ -510,13 +542,32 @@ set_style(s, v)
 int	s;
 double	v;
 {
-	v /= POINT_PER_INCH / (double)resolution;
+	v /= 80.0 / (double)resolution;
 	if (s == DASH_LINE) {
-	    if (v > 0.0) fprintf(tfp, " [%.1f] 0 sd\n", v);
+	    if (v > 0.0) fprintf(tfp, " [%d] 0 sd\n", round(v));
 	    }
 	else if (s == DOTTED_LINE) {
-	    if (v > 0.0) fprintf(tfp, " [%d %.1f] %.1f sd\n", 
-		round(resolution/80.0), v, v);
+	    if (v > 0.0) fprintf(tfp, " [%d %d] %d sd\n", 
+		round(resolution/80.0), round(v), round(v));
+	    }
+	else if (s == DASH_DOT_LINE) {
+	    if (v > 0.0) fprintf(tfp, " [%d %d %d %d] 0 sd\n", 
+		round(v), round(v*0.5),
+		round(resolution/80.0), round(v*0.5));
+	    }
+	else if (s == DASH_2_DOTS_LINE) {
+	    if (v > 0.0) fprintf(tfp, " [%d %d %d %d %d %d] 0 sd\n", 
+		round(v), round(v*0.45),
+		round(resolution/80.0), round(v*0.333),
+		round(resolution/80.0), round(v*0.45));
+	    }
+	else if (s == DASH_3_DOTS_LINE) {
+	    if (v > 0.0) fprintf(tfp, 
+                " [%d %d %d %d %d %d %d %d ] 0 sd\n", 
+		round(v), round(v*0.4),
+		round(resolution/80.0), round(v*0.3),
+		round(resolution/80.0), round(v*0.3),
+		round(resolution/80.0), round(v*0.4));
 	    }
 	}
 
@@ -531,6 +582,10 @@ double	v;
 	else if (s == DOTTED_LINE) {
 	    if (v > 0.0) fprintf(tfp, " [] 0 sd");
 	    }
+	else if (s == DASH_DOT_LINE || s == DASH_2_DOTS_LINE ||
+                 s == DASH_3_DOTS_LINE) {
+	    if (v > 0.0) fprintf(tfp, " [] 0 sd");
+	    }
 	fprintf(tfp, "\n");
 	}
 
@@ -538,8 +593,6 @@ static
 set_linejoin(j)
 int	j;
 {
-	extern int	cur_joinstyle;
-
 	if (j != cur_joinstyle) {
 	    cur_joinstyle = j;
 	    fprintf(tfp, "%d slj\n", cur_joinstyle);
@@ -550,8 +603,6 @@ static
 set_linecap(j)
 int	j;
 {
-	extern int	cur_capstyle;
-
 	if (j != cur_capstyle) {
 	    cur_capstyle = j;
 	    fprintf(tfp, "%d slc\n", cur_capstyle);
@@ -589,20 +640,28 @@ F_line	*l;
 	int		xmin,xmax,ymin,ymax;
 	int		pic_w, pic_h;
 	Boolean		namedcol;
+	float		hf_wid;
 	
 	if (multi_page)
 	   fprintf(tfp, "/o%d {", no_obj++);
+	fprintf(tfp, "%% Polyline\n");
 	if (l->type != T_PIC_BOX) {  /* pic object has no line thickness */
 		set_linejoin(l->join_style);
 		set_linecap(l->cap_style);
 		set_linewidth((double)l->thickness);
 	}
-	fprintf(tfp, "%% Polyline\n");
 	p = l->points;
 	q = p->next;
 	if (q == NULL) { /* A single point line */
+	    /*set_linecap(0);	/* make CAP_BUTT */
+	    if (l->cap_style > 0)
+		hf_wid = 1.0;
+	    else if (l->thickness <= THICK_SCALE)
+		hf_wid = l->thickness/4.0;
+	    else
+		hf_wid = (l->thickness-THICK_SCALE)/2.0;
 	    fprintf(tfp, "n %d %d m %d %d l gs col%d s gr\n",
-			p->x, p->y, p->x, p->y, l->pen_color);
+			round(p->x-hf_wid), p->y, round(p->x+hf_wid), p->y, l->pen_color);
 	    if (multi_page)
 	       fprintf(tfp, "} bind def\n");
 	    return;
@@ -643,8 +702,7 @@ F_line	*l;
 				xmax, ymax, xmax, ymin+radius, radius);
 	    fprintf(tfp, "  %d %d %d %d %d arcto 4 {pop} repeat\n", /* arc through tr to tl */
 				xmax, ymin, xmin+radius, ymin, radius);
-	}
-	else if (l->type == T_PIC_BOX) {  /* postscript (eps), XPM, X bitmap or GIF file */
+	} else if (l->type == T_PIC_BOX) {  /* postscript (eps), XPM, X bitmap or GIF file */
 	  /* PICTURE OBJECT */
 		int             dx, dy, rotation;
 		int		llx, lly, urx, ury;
@@ -698,6 +756,7 @@ F_line	*l;
 			ury = l->pic->bit_size.y;
 
 		/* not GIF, try JPEG (JFIF) */
+#ifdef USE_JPEG
 		} else if (read_jpg(l->pic) > 0) {
 			/* yes, say so */
 			fprintf(tfp, "%% Begin Imported JPEG File: %s\n", l->pic->file);
@@ -706,14 +765,15 @@ F_line	*l;
 			urx = l->pic->bit_size.x;	/* size of image from the file */
 			ury = l->pic->bit_size.y;
 
+#endif /* USE_JPEG */
 		/* none of the above, try EPS */
 		} else {
 			fprintf(tfp, "%% Begin Imported EPS File: %s\n", l->pic->file);
-			fprintf(tfp, "%%BeginDocument: %s\n", l->pic->file);
+			fprintf(tfp, "%%%%BeginDocument: %s\n", l->pic->file);
 			fprintf(tfp, "%%\n");
 
 			if ((picf=open_picfile(l->pic->file, &filtype)) == NULL) {
-			    fprintf(stderr, "Unable to open EPS file: %s, error: (%d)\n",
+			    fprintf(stderr, "Unable to open EPS file '%s': error: %s (%d)\n",
 					l->pic->file, sys_errlist[errno],errno);
 			    return;
 			}
@@ -721,9 +781,10 @@ F_line	*l;
 			    char *c;
 
 			    if (!strncmp(buf, "%%BoundingBox:", 14)) {
-				switch (*(c=buf+14)) {
-				    case ' ':case '\t':c++;
-				}
+				c=buf+14;
+				/* skip past white space */
+				while (*c == ' ' || *c == '\t') 
+				    c++;
 				if (strncmp(c,"(atend)",7)) {	/* make sure not an (atend) */
 				    if (sscanf(c, "%lf %lf %lf %lf",
 						&fllx, &flly, &furx, &fury) < 4) {
@@ -812,6 +873,17 @@ F_line	*l;
 		fprintf(tfp, "%d %d tr\n", -llx, -lly);
 		/* save vm so pic file won't change anything */
 		fprintf(tfp, "sa\n");
+
+		/* if PIC object is EPS file, set up clipping rectangle to BB
+		 * and prepare to clean up stacks and dicts of included EPS file
+		 */
+		if (l->pic->subtype == P_EPS) {
+		    fprintf(tfp, "n %d %d m %d %d l %d %d l %d %d l cp clip\n",
+			llx,lly, urx,lly, urx,ury, llx,ury);
+		    fprintf(tfp, "countdictstack\n");
+		    fprintf(tfp, "mark\n");
+		}
+
 		/* and undefine showpage */
 		fprintf(tfp, "/showpage {} def\n");
 
@@ -866,12 +938,12 @@ F_line	*l;
 			fprintf(tfp, "%d %d sc\n", urx, ury);
 			/* modify colortable entries to make consistent */
 			coltabl = xpmimage.colorTable;
-			namedcol = False;
+			namedcol = FALSE;
 			/* convert the color defs to a consistent #rrggbb */
 			for (i=0; i<xpmimage.ncolors; i++) {
 				c = (coltabl + i)->c_color;
 				if (c[0] != '#') {		/* named color, set flag */
-					namedcol = True;
+					namedcol = TRUE;
 				/* want to make #RRGGBB from possibly other formats */
 				} else if (strlen(c) == 4) {	/* #rgb */
 					sprintf(tmpc,"#%.1s%.1s%.1s%.1s%.1s%.1s",
@@ -908,7 +980,7 @@ F_line	*l;
 			for (i=0; i<wid*ht; i++)
 			    *cp++ = (unsigned char) *dp++;
 				
-			/* now write out the compressed image data */
+			/* now write out the image data in a compressed form */
 			(void) PSencode(tfp, wid, ht, xpmimage.ncolors,
 				l->pic->cmap[0], l->pic->cmap[1], l->pic->cmap[2], 
 				cdata);
@@ -930,7 +1002,7 @@ F_line	*l;
 			    fprintf(tfp, "%% JPEG image follows:\n");
 			/* scale for size in bits */
 			fprintf(tfp, "%d %d sc\n", urx, ury);
-			/* now write out the compressed image data */
+			/* now write out the image data in a compressed form */
 			(void) PSencode(tfp, wid, ht, l->pic->numcols,
 				l->pic->cmap[0], l->pic->cmap[1], l->pic->cmap[2], 
 				l->pic->bitmap);
@@ -940,7 +1012,7 @@ F_line	*l;
 		    int i;
 		    fprintf(tfp, "%% EPS file follows:\n");
 		    if ((picf=open_picfile(l->pic->file, &filtype)) == NULL) {
-			fprintf(stderr, "Unable to open EPS file: %s, error: (%d)\n",
+			fprintf(stderr, "Unable to open EPS file '%s': error: %s (%d)\n",
 				l->pic->file, sys_errlist[errno],errno);
 			fprintf(tfp, "gr\n");
 			return;
@@ -954,11 +1026,19 @@ F_line	*l;
 		    close_picfile(picf,filtype);
 		}
 
+		/* if PIC object is EPS file, clean up stacks and dicts
+		 * before 'restore'ing vm
+		 */
+		if (l->pic->subtype == P_EPS) {
+		    fprintf(tfp, "cleartomark\n");
+		    fprintf(tfp, "countdictstack exch sub { end } repeat\n");
+		}
+
 		/* restore vm and gsave */
-		fprintf(tfp, "rs gr\n");
+		fprintf(tfp, "restore grestore\n");
 		fprintf(tfp, "%%\n");
 		fprintf(tfp, "%% End Imported PIC File: %s\n", l->pic->file);
-		fprintf(tfp, "%%EndDocument\n");
+		fprintf(tfp, "%%%%EndDocument\n");
 		fprintf(tfp, "%%\n");
 	} else {
 	  /* POLYLINE */
@@ -1000,12 +1080,17 @@ F_line	*l;
 			fprintf(tfp, "\n");
 		}
 	}
+
 	/* now fill it, draw the line and/or draw arrow heads */
 	if (l->type != T_PIC_BOX) {	/* make sure it isn't a picture object */
-		if (l->type == T_POLYLINE)
+		if (l->type == T_POLYLINE) {
 		    fprintf(tfp, " %d %d l ", q->x, q->y);
-		else
+		    if (fpntx1==lpntx1 && fpnty1==lpnty1)
+			fprintf(tfp, " cp ");	/* endpoints are coincident, close path 
+							so that line join is used */
+		} else {
 		    fprintf(tfp, " cp ");	/* polygon, close path */
+		}
 		/* fill it if there is a fill style */
 		if (l->fill_style != UNFILLED)
 		    fill_area(l->fill_style, l->pen_color, l->fill_color, xmin, ymin);
@@ -1039,6 +1124,8 @@ F_spline	*s;
 	} else {
 	    set_linecap(s->cap_style);	/* open splines can explicitely set capstyle */
 	}
+	/* set the line thickness */
+	set_linewidth((double)s->thickness);
 	if (int_spline(s))
 	    genps_itp_spline(s);
 	else
@@ -1054,7 +1141,6 @@ F_spline	*s;
 	F_control	*a, *b, *ar;
 	int		 xmin, ymin;
 
-	set_linewidth((double)s->thickness);
 	fprintf(tfp, "%% Interp Spline\n");
 	a = ar = s->controls;
 
@@ -1168,7 +1254,6 @@ F_spline	*s;
 	    clip_arrows(s, O_SPLINE);
 
 	/* now output the points */
-	set_linewidth((double)s->thickness);
 	set_style(s->style, s->style_val);
 	xmin = 999999;
 	ymin = 999999;
@@ -1237,6 +1322,9 @@ F_arc	*a;
 
 	if (multi_page)
 	   fprintf(tfp, "/o%d {", no_obj++);
+
+	fprintf(tfp, "%% Arc\n");
+
 	cx = a->center.x; cy = a->center.y;
 	sx = a->point[0].x; sy = a->point[0].y;
 	ex = a->point[2].x; ey = a->point[2].y;
@@ -1247,7 +1335,6 @@ F_arc	*a;
 	    direction = a->direction;
 	set_linewidth((double)a->thickness);
 	set_linecap(a->cap_style);
-	fprintf(tfp, "%% Arc\n");
 	dx = cx - sx;
 	dy = cy - sy;
 	radius = sqrt(dx*dx+dy*dy);
@@ -1323,6 +1410,8 @@ F_ellipse	*e;
 	    fprintf(tfp, "%6.3f rot\n",-e->angle*180/M_PI);
 	    fprintf(tfp, "n 0 0 %d %d 0 360 DrawEllipse ",
 		 e->radiuses.x, e->radiuses.y);
+	    /* rotate back so any fill pattern will come out correct */
+	    fprintf(tfp, "%6.3f rot\n",e->angle*180/M_PI);
 	}
 	if (e->fill_style != UNFILLED)
 	    fill_area(e->fill_style, e->pen_color, e->fill_color,
@@ -1345,9 +1434,26 @@ genps_text(t)
 F_text	*t;
 {
 	unsigned char		*cp;
+#ifdef I18N
+#define LINE_LENGTH_LIMIT 200
+	Boolean composite = FALSE;
+	Boolean state_gr = FALSE;
+	int chars = 0;
+        int gr_chars = 0;
+	unsigned char ch;
+#endif /* I18N */
 
 	if (multi_page)
 	   fprintf(tfp, "/o%d {", no_obj++);
+#ifdef I18N
+	if (enable_composite_font && (t->font <= 0 || t->font == 2)) {
+	  composite = TRUE;
+	  if (t->font <= 0)
+	    fprintf(tfp, TEXT_PS, "CompositeRoman", "", PSFONTMAG(t));
+	  else
+	    fprintf(tfp, TEXT_PS, "CompositeBold", "", PSFONTMAG(t));
+	} else
+#endif /* I18N */
 	if (PSisomap[t->font+1] == TRUE)
 	   fprintf(tfp, TEXT_PS, PSFONT(t), "-iso", PSFONTMAG(t));
 	else
@@ -1361,6 +1467,55 @@ F_text	*t;
 	   fprintf(tfp, " %.1f rot ", t->angle*180/M_PI);
 	/* this loop escapes characters '(', ')', and '\' */
 	fputc('(', tfp);
+#ifdef I18N
+	for(cp = (unsigned char *)t->cstring; *cp; cp++) {
+	    if (LINE_LENGTH_LIMIT < chars) {
+	      fputs("\\\n", tfp);
+	      chars = 0;
+	    }
+	    ch = *cp;
+	    if (enable_composite_font && composite) {
+	      if (ch & 0x80) {  /* GR */
+		if (!state_gr) {
+		  fprintf(tfp, "\\377\\001");
+		  chars += 8;
+		  state_gr = TRUE;
+		  gr_chars = 0;
+		}
+		gr_chars++;
+	      } else {  /* GL */
+		if (state_gr) {
+		  if (gr_chars % 2) {
+		    fprintf(stderr, "warning: incomplete multi-byte text: %s\n",
+			    t->cstring);
+		    fputc('?', tfp);
+		  }
+		  fprintf(tfp, "\\377\\000");
+		  chars += 8;
+		  state_gr = FALSE;
+		}
+	      }
+            }
+	    if (strchr("()\\", ch)) {
+	      fputc('\\', tfp);
+	      chars += 1;
+	    }
+	    if (ch>=0x80) {
+	      fprintf(tfp,"\\%o", ch);
+	      chars += 4;
+	    } else {
+	      fputc(ch, tfp);
+	      chars += 1;
+	    }
+	  }
+	  if (enable_composite_font && composite && state_gr) {
+	    if (gr_chars % 2) {
+	      fprintf(stderr, "warning: incomplete multi-byte text: %s\n",
+		      t->cstring);
+	      fputc('?', tfp);
+	    }
+	  }
+#else
 	for(cp = (unsigned char *)t->cstring; *cp; cp++) {
 	    if (strchr("()\\", *cp))
 		fputc('\\', tfp);
@@ -1369,6 +1524,7 @@ F_text	*t;
 	    else
 		fputc(*cp, tfp);
 		}
+#endif /* I18N */
 	fputc(')', tfp);
 
 	if ((t->type == T_CENTER_JUSTIFIED) || (t->type == T_RIGHT_JUSTIFIED)){
@@ -1466,9 +1622,8 @@ clip_arrows(obj, objtype)
 	for (i=0; i<3; i++) {
 	    fprintf(tfp,"%d %d %c ",fpoints[i].x,fpoints[i].y,i==0? 'm':'l');
 	}
-	fprintf(tfp, "%d %d l %d %d l ",fcx2, fcy2, fcx1, fcy1);
-	/* intersect this with current clip path */
-	fprintf(tfp, " cp clip\n");
+	/* clip path for the forward arrow */
+	fprintf(tfp, "%d %d l %d %d l cp\n",fcx2, fcy2, fcx1, fcy1);
     }
 	
     /* get points for any backward arrowhead */
@@ -1490,10 +1645,11 @@ clip_arrows(obj, objtype)
 	for (i=0; i<3; i++) {
 	    fprintf(tfp,"%d %d %c ",bpoints[i].x,bpoints[i].y,i==0? 'm':'l');
 	}
-	fprintf(tfp, "%d %d l %d %d l ",bcx2, bcy2, bcx1, bcy1);
-	/* intersect this with current clip path */
-	fprintf(tfp, " cp clip\n");
+	/* clip path for the backward arrow */
+	fprintf(tfp, "%d %d l %d %d l cp\n",bcx2, bcy2, bcx1, bcy1);
     }
+    /* intersect the arrowhead clip path(s) with current clip path */
+    fprintf(tfp, "clip\n");
 }
 
 /****************************************************************
@@ -1531,8 +1687,13 @@ calc_arrow(x1, y1, x2, y2, c1x, c1y, c2x, c2y, objthick, arrow, points, npoints)
     int		    style = arrow->style;
 
     *npoints = 0;
+    *c1x=x1;
+    *c1y=y1;
+    *c2x=x2;
+    *c2y=y2;
     dx = x2 - x1;
     dy = y1 - y2;
+    /* two points are the same, can't determine an arrowhead */
     if (dx==0 && dy==0)
 	return;
 
@@ -1616,16 +1777,17 @@ calc_arrow(x1, y1, x2, y2, c1x, c1y, c2x, c2y, objthick, arrow, points, npoints)
 
 static
 arc_tangent(x1, y1, x2, y2, direction, x, y)
-double	x1, y1, x2, y2, *x, *y;
+double	x1, y1, x2, y2;
+int	*x, *y;
 int	direction;
 {
-	if (direction) { /* counter clockwise  */
-	    *x = x2 + (y2 - y1);
-	    *y = y2 - (x2 - x1);
+	if (direction==0) { /* counter clockwise  */
+	    *x = round(x2 + (y2 - y1));
+	    *y = round(y2 - (x2 - x1));
 	    }
 	else {
-	    *x = x2 - (y2 - y1);
-	    *y = y2 + (x2 - x1);
+	    *x = round(x2 - (y2 - y1));
+	    *y = round(y2 + (x2 - x1));
 	    }
 	}
 
@@ -1648,22 +1810,23 @@ compute_arcarrow_angle(x1, y1, x2, y2, direction, arrow, x, y)
     dy=y2-y1;
     dx=x2-x1;
     r=sqrt(dx*dx+dy*dy);
-    if (arrow->ht>2*r) {
-	arc_tangent(x1,y1,x2,y2,direction,x,y);
-	return;
-    }
-
     h = (double) arrow->ht;
     /* lpt is the amount the arrowhead extends beyond the end of the line */
     lpt = arrow->thickness/2.0/(arrow->wid/h/2.0);
     /* add this to the length */
     h += lpt;
 
+    /* radius too small for this method, use normal method */
+    if (h > 2.0*r) {
+	arc_tangent(x1,y1,x2,y2,direction,x,y);
+	return;
+    }
+
     beta=atan2(dy,dx);
     if (direction) {
-	alpha=2*asin(h/2.0/r);
+	alpha = 2*asin(h/2.0/r);
     } else {
-	alpha=-2*asin(h/2.0/r);
+	alpha = -2*asin(h/2.0/r);
     }
 
     *x=round(x1+r*cos(beta+alpha));
@@ -1680,7 +1843,7 @@ int fill, pen_color, fill_color, ulx, uly;
    float pen_r, pen_g, pen_b, fill_r, fill_g, fill_b;
 
    /* get the rgb values for the fill pattern (if necessary) */
-   if (fill_color <= NUM_STD_COLS) {
+   if (fill_color < NUM_STD_COLS) {
 	fill_r=rgbcols[fill_color>0? fill_color: 0].r;
 	fill_g=rgbcols[fill_color>0? fill_color: 0].g;
 	fill_b=rgbcols[fill_color>0? fill_color: 0].b;
@@ -1689,7 +1852,7 @@ int fill, pen_color, fill_color, ulx, uly;
 	fill_g=user_colors[fill_color-NUM_STD_COLS].g/255.0;
 	fill_b=user_colors[fill_color-NUM_STD_COLS].b/255.0;
    }
-   if (pen_color <= NUM_STD_COLS) {
+   if (pen_color < NUM_STD_COLS) {
 	pen_r=rgbcols[pen_color>0? pen_color: 0].r;
 	pen_g=rgbcols[pen_color>0? pen_color: 0].g;
 	pen_b=rgbcols[pen_color>0? pen_color: 0].b;
@@ -1767,16 +1930,18 @@ F_compound      *ob;
       {
 	 for (s = (unsigned char*)t->cstring; *s != '\0'; s++)
 	 {
-	    /* look for characters >= 128 */
-	    if (*s>127) return(1);
+	    /* look for characters >= 128 or ASCII '-' */
+	    if ((*s>127) || (*s==45))
+		return 1;
 	 }
       }
    }
 
    for (c = ob->compounds; c != NULL; c = c->next) {
-       if (iso_text_exist(c)) return(1);
-       }
-   return(0);
+	if (iso_text_exist(c))
+	    return 1;
+   }
+   return 0;
 }
 
 static
@@ -1786,18 +1951,15 @@ F_compound	*ob;
    F_compound *c;
    F_text     *t;
 
-   if (ob->texts != NULL)
-   {
+   if (ob->texts != NULL) {
 	for (t = ob->texts; t != NULL; t = t->next)
-	    if (PSisomap[t->font+1] == FALSE)
-	    {
+	    if (PSisomap[t->font+1] == FALSE) {
 		fprintf(tfp, "/%s /%s-iso isovec ReEncode\n", PSFONT(t), PSFONT(t));
 		PSisomap[t->font+1] = TRUE;
-	    }
+	}
    }
 
-   for (c = ob->compounds; c != NULL; c = c->next)
-   {
+   for (c = ob->compounds; c != NULL; c = c->next) {
 	encode_all_fonts(c);
    }
 }
@@ -1808,31 +1970,35 @@ F_compound	*ob;
 {
 	F_compound	*c;
 
-	if (NULL != ob->ellipses) return(1);
+	if (NULL != ob->ellipses) 
+		return 1;
 
 	for (c = ob->compounds; c != NULL; c = c->next) {
-	    if (ellipse_exist(c)) return(1);
-	    }
+	    if (ellipse_exist(c))
+		return 1;
+	}
 
-	return(0);
+	return 0;
 	}
 
 static
-normal_spline_exist(ob)
+approx_spline_exist(ob)
 F_compound	*ob;
 {
 	F_spline	*s;
 	F_compound	*c;
 
 	for (s = ob->splines; s != NULL; s = s->next) {
-	    if (normal_spline(s)) return(1);
-	    }
+	    if (approx_spline(s))
+		return 1;
+	}
 
 	for (c = ob->compounds; c != NULL; c = c->next) {
-	    if (normal_spline_exist(c)) return(1);
-	    }
+	    if (approx_spline_exist(c))
+		return 1;
+	}
 
-	return(0);
+	return 0;
 	}
 
 struct
