@@ -54,21 +54,6 @@
 #include "object.h"
 #include "texfonts.h"
 
-#ifndef fabs
-extern double fabs();
-#endif
-#ifndef sin
-extern double sin();
-#endif
-#ifndef cos
-extern double cos();
-#endif
-#ifndef acos
-extern double acos();
-#endif
-#ifndef atan
-extern double atan();
-#endif
 extern double rad2deg;
 
 #ifdef hpux
@@ -127,7 +112,7 @@ F_text	*t;
 {
 
 	if (!special_text(t))
-	  gendev_null(t);
+	  gendev_null();
 	else genlatex_text(t);
 }
 
@@ -137,7 +122,7 @@ F_text	*t;
 
 	if (!special_text(t))
 	  genps_text(t);
-	else gendev_null(t);
+	else gendev_null();
 }
 
 void genpstex_option(opt, optarg)
