@@ -33,7 +33,6 @@
 #endif
 #include <sys/file.h>
 #include <stdio.h>
-#include <ctype.h>
 #include "patchlevel.h"
 #include "fig2dev.h"
 #include "object.h"
@@ -42,10 +41,16 @@
 extern int fig_getopt();
 extern char *optarg;
 extern int optind;
+char	lang[40];
 
 struct driver *dev = NULL;
 
+#ifdef I18N
+char		Usage[] = "Usage: %s [-L language] [-f font] [-s size] [-m scale] [-j] [input [output]]\n";
+Boolean support_i18n = FALSE;
+#else
 char		Usage[] = "Usage: %s [-L language] [-f font] [-s size] [-m scale] [input [output]]\n";
+#endif  /* I18N */
 char		Err_badarg[] = "Argument -%c unknown to %s driver.";
 char		Err_incomp[] = "Incomplete %s object at line %d.";
 char		Err_mem[] = "Running out of memory.";
@@ -57,10 +62,20 @@ int		font_size = 0;
 double		mag = 1.0;
 FILE		*tfp = NULL;
 int		llx = 0, lly = 0, urx = 0, ury = 0;
-int		landscape;
-int		center;
-int		orientspec=0;		/* set it the user-specifies the orientation */
+Boolean		landscape;
+Boolean		center;
+Boolean		orientspec=FALSE;	/* set if the user specifies the orientation */
+Boolean		centerspec=FALSE;	/* set if the user specifies the justification */
+Boolean		magspec=FALSE;		/* set if the user specifies the magnification */
+Boolean		multispec=FALSE;	/* set if the user specifies multiple pages */
+Boolean		paperspec=FALSE;	/* set if the user specifies the paper size */
 Boolean		pats_used, pattern_used[NUMPATTERNS];
+Boolean		multi_page = FALSE;	/* multiple page option for PostScript */
+Boolean		metric;			/* true if file specifies Metric */
+char		papersize[20];		/* paper size */
+float		THICK_SCALE;		/* convert line thickness from screen res. */
+					/* calculated in read_objects() */
+char		lang[40];		/* selected output language */
 
 struct obj_rec {
 	void (*gendev)();
@@ -68,6 +83,29 @@ struct obj_rec {
 	int depth;
 };
 
+/* be sure to update NUMPAPERSIZES in fig2dev.h if this table changes */
+
+struct paperdef paperdef[] =
+{
+    {"Letter", 612, 792}, 	/* 8.5" x 11" */
+    {"Legal", 612, 1008}, 	/* 8.5" x 14" */
+    {"Ledger", 1224, 792}, 	/*  17" x 11" */
+    {"Tabloid", 792, 1224}, 	/*  11" x 17" */
+    {"A", 612, 792},		/* 8.5" x 11" (letter) */
+    {"B", 792, 1224},		/*  11" x 17" (tabloid) */
+    {"C", 1224, 1584},		/*  17" x 22" */
+    {"D", 1584, 2448},		/*  22" x 34" */
+    {"E", 2448, 3168},		/*  34" x 44" */
+    {"B5", 516, 729}, 		/* 18.2cm x  25.7cm */
+    {"A4", 595, 842}, 		/* 21  cm x  29.7cm */
+    {"A3", 842, 1190},		/* 29.7cm x  42  cm */
+    {"A2", 1190, 1684},		/* 42  cm x  59.4cm */
+    {"A1", 1684, 2383},		/* 59.4cm x  84.1cm */
+    {"A0", 2383, 3370},		/* 84.1cm x 118.9cm */ 
+    {NULL, 0, 0}
+};
+
+
 put_msg(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8)
 char   *format, *arg1, *arg2, *arg3, *arg4, *arg5, *arg6, *arg7, *arg8;
 {
@@ -85,7 +123,11 @@ char	*argv[];
 	prog = *argv;
 /* add :? */
 	/* sum of all arguments */
-	while ((c = fig_getopt(argc, argv, "acC:d:ef:l:L:Mm:n:Pp:s:S:t:vVx:X:y:Y:wWz:?")) != EOF) {
+#ifdef I18N
+	while ((c = fig_getopt(argc, argv, "aAcC:d:ef:l:L:Mm:n:q:Pp:s:S:t:vVx:X:y:Y:wWz:j?")) != EOF) {
+#else
+	while ((c = fig_getopt(argc, argv, "aAcC:d:ef:l:L:Mm:n:q:Pp:s:S:t:vVx:X:y:Y:wWz:?")) != EOF) {
+#endif
 
 	  /* generic option handling */
 	  switch (c) {
@@ -97,12 +139,14 @@ char	*argv[];
 			break;
 
 		case 'L':			/* set output language */
+		    /* save language for gen{gif,jpg,pcx,xbm,xpm,ppm,tif} */
+		    strncpy(lang,optarg,sizeof(lang)-1);
 		    for (i=0; *drivers[i].name; i++) 
-			if (!strcmp(optarg, drivers[i].name))
+			if (!strcmp(lang, drivers[i].name))
 				dev = drivers[i].dev;
 		    if (!dev) {
 			fprintf(stderr,
-				"Unknown graphics language %s\n", optarg);
+				"Unknown graphics language %s\n", lang);
 			fprintf(stderr,"Known languages are:\n");
 			/* display available languages - 23/01/90 */
 			for (i=0; *drivers[i].name; i++)
@@ -122,8 +166,14 @@ char	*argv[];
 
 		case 'm':			/* set magnification */
 		    mag = atof(optarg);
+		    magspec = TRUE;		/* user-specified */
 		    break;
 
+#ifdef I18N
+		case 'j':			/* set magnification */
+		    support_i18n = TRUE;
+		    continue;  /* don't pass this option to driver */
+#endif /* I18N */
 		case '?':			/* usage 		*/
 			fprintf(stderr,Usage,prog);
 			exit(1);
@@ -175,7 +225,8 @@ char	*argv[];
 	    }
 
 	gendev_objects(&objects, dev);
-	if (tfp != stdout) (void)fclose(tfp);
+	if ((tfp != stdout) && (tfp != 0)) 
+	    (void)fclose(tfp);
 	exit(0);
 	}
 
@@ -260,7 +311,7 @@ struct driver *dev;
 	/* dump object pointers to an array */
 	obj_count = compound_dump(objects, 0, 0, dev);
 	if (!obj_count) {
-	    fprintf(stderr, "No object");
+	    fprintf(stderr, "No object\n");
 	    return;
 	    }
 	rec_array = (struct obj_rec *)malloc(obj_count*sizeof(struct obj_rec));
@@ -278,6 +329,7 @@ struct driver *dev;
 
 	/* generate trailer */
 	(*dev->end)();
+	free(rec_array);
 }
 
 int rec_comp(r1, r2)
