@@ -31,6 +31,15 @@ typedef char Boolean;
 
 #define DEFAULT_FONT_SIZE 11
 
+#ifdef USE_INLINE
+#define INLINE inline
+#else
+#define INLINE
+#endif /* USE_INLINE */
+
+/* include ctype.h for isascii() and isxdigit() macros */
+#include <ctype.h>
+
 #if defined(SYSV) || defined(SVR4)
 #include <string.h>
 #else
@@ -38,7 +47,7 @@ typedef char Boolean;
 #ifndef strchr
 extern char *strchr();
 #endif
-#ifndef NeXT
+#if !defined(NeXT) && !defined(SYSV) && !defined(SVR4)
 #define	strchr	index
 #define	strrchr	rindex
 #endif
@@ -80,16 +89,35 @@ extern char *PSfontnames[];
 
 extern int   PSisomap[];
 
-extern char	*prog, *from;
+extern char	*prog, *from, *to;
 extern char	*name;
 extern int	font_size;
 extern double	mag;
 extern FILE	*tfp;
 
 extern int	llx, lly, urx, ury;
-extern int	landscape;
-extern int	center;
-extern int	orientspec;	/* true of the command-line args specified land or port */
+extern Boolean	landscape;
+extern Boolean	center;
+extern Boolean	multi_page;	/* multiple page option for PostScript */
+extern Boolean	orientspec;	/* true if the command-line args specified land or port */
+extern Boolean	centerspec;	/* true if the command-line args specified -c or -e */
+extern Boolean	magspec;	/* true if the command-line args specified -m */
+extern Boolean	paperspec;	/* true if the command-line args specified -z */
+extern Boolean	multispec;	/* true if the command-line args specified -M */
+extern Boolean	metric;		/* true if the file contains Metric specifier */
+extern char	papersize[];	/* paper size */
+extern float	THICK_SCALE;	/* convert line thickness from screen res. */
+extern char	lang[];		/* selected output language */
+
+struct paperdef
+{
+    char *name;			/* name for paper size */
+    int width;			/* paper width in points */
+    int height;			/* paper height in points */
+};
+
+#define NUMPAPERSIZES 15
+extern struct paperdef paperdef[];
 
 /* user-defined colors */
 typedef		struct{
@@ -111,3 +139,27 @@ struct Cmap {
 	unsigned short red, green, blue;
 	unsigned long pixel;
 };
+
+/* define PATH_MAX if not already defined */
+/* taken from the X11R5 server/os/osfonts.c file */
+#ifndef X_NOT_POSIX
+#ifdef _POSIX_SOURCE
+#include <limits.h>
+#else
+#if !defined(sun) || defined(sparc)
+#define _POSIX_SOURCE
+#include <limits.h>
+#undef _POSIX_SOURCE
+#endif /* !defined(sun) || defined(sparc) */
+#endif /* _POSIX_SOURCE */
+#endif /* X_NOT_POSIX */
+
+#ifndef PATH_MAX
+#include <sys/param.h>
+#ifdef MAXPATHLEN
+#define PATH_MAX MAXPATHLEN
+#else
+#define PATH_MAX 1024
+#endif /* MAXPATHLEN */
+#endif /* PATH_MAX */
+
