@@ -1,3 +1,5 @@
+/*	BSDI $Id$	*/
+
 /*
  * TransFig: Facility for Translating Fig code
  * Copyright (c) 1985 Supoj Sutantavibul
@@ -37,15 +39,30 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
+#include <sys/param.h>
+#include <string.h>
 #include "alloc.h"
 #include "fig2dev.h"
 #include "object.h"
+#include "trans_spline.h"
 #include "../patchlevel.h"
 
 #if defined(hpux) || defined(SYSV) || defined(SVR4)
 #define bzero(s,n) memset((s),'\0',(n))
 #endif
 
+#ifdef NEED_STRERROR
+char *
+strerror(e)
+	int e;
+{
+#if (! (defined(BSD) && (BSD >= 199306)) && !defined(__NetBSD__))
+	extern char *sys_errlist[];
+#endif
+	return sys_errlist[e];
+	}
+#endif
+
 extern int            errno;
 
 extern F_arrow		*make_arrow();
@@ -75,14 +92,12 @@ int			num_object;
 int			v2_flag;	/* Protocol V2.0 or higher */
 int			v21_flag;	/* Protocol V2.1 or higher */
 int			v30_flag;	/* Protocol V3.0 or higher */
-float			THICK_SCALE;	/* convert line thickness from screen res. */
+int			v32_flag;	/* Protocol V3.2 or higher */
 
 read_fail_message(file, err)
 char	*file;
 int	err;
 {
-	extern char	*sys_errlist[];
-
 	if (err == 0)		/* Successful read */
 	    return;
 #if !defined(hpux) && !defined(SYSV) && !defined(SVR4)
@@ -104,7 +119,7 @@ int	err;
 	    /* Format error; relevant error message is already delivered */
 	    }
 	else
-	    put_msg("File \"%s\" is not accessable; %s", file, sys_errlist[err]);
+	    put_msg("File \"%s\" is not accessable; %s", file, strerror(err));
 	}
 
 /**********************************************************
@@ -127,9 +142,9 @@ F_compound	*obj;
 
 	init_pats_used();
 	if ((fp = fopen(file_name, "r")) == NULL)
-	    return(errno);
+	    return errno;
 	else
-	    return(readfp_fig(fp, obj));
+	    return readfp_fig(fp, obj);
 	}
 
 readfp_fig(fp, obj)
@@ -142,15 +157,15 @@ F_compound	*obj;
 	num_object = 0;
 	num_usr_cols = 0;
 	c = fgetc(fp);
-	if (feof(fp)) return(-2);
-	ungetc(c, fp);
+	if (feof(fp)) 
+	    return -2;
 	bzero((char*)obj, COMOBJ_SIZE);
 	if (c == '#')
 	    status = read_objects(fp, obj);
 	else
 	    status = read_1_3_objects(fp, obj);
 	(void)fclose(fp);
-	return(status);
+	return status;
 	}
 	
 int
@@ -167,8 +182,10 @@ F_compound	*obj;
 	int		object, ppi, coord_sys;
 
 	bzero((char*)obj, COMOBJ_SIZE);
+	/* get back to the top */
+	rewind(fp);
 	(void)fgets(buf, BUF_SIZE, fp);	/* get the version line */
-	if (strlen(buf) > 0)
+	if (strlen(buf) > (size_t)0)
 	    buf[strlen(buf)-1] = '\0';	/* remove newline */
 
 	/* v2_flag is for version 2 or higher */
@@ -177,6 +194,9 @@ F_compound	*obj;
 	v21_flag = (!strncmp(buf, "#FIG 2.1", 8) || !strncmp(buf, "#FIG 3", 6));
 	/* version 2.2 was only beta - 3.0 is the official release (they are identical) */
 	v30_flag = (!strncmp(buf, "#FIG 3", 6) || !strncmp(buf, "#FIG 2.2", 8));
+	/* version 3.2 contains paper size, magnif, multiple page and transparent color
+	   in Fig file */
+	v32_flag = (!strncmp(buf, "#FIG 3.2", 8));
 	if (strncmp(&buf[5],VERSION,3) > 0) {
 	    put_msg("Fig file format (%s) newer than this version of fig2dev (%s), exiting",
 			&buf[5],VERSION);
@@ -188,9 +208,9 @@ F_compound	*obj;
 	    line_no++;
 	    if (get_line(fp) < 0) {
 		put_msg("File is truncated at landscape/portrait specification.");
-		return(-1);
+		return -1;
 	    }
-	    /* but set only of the user didn't specify the orientation
+	    /* but set only if the user didn't specify the orientation
 	       on the command line */
 	    if (!orientspec)
 		landscape = !strncasecmp(buf,"land",4);
@@ -199,32 +219,98 @@ F_compound	*obj;
 	    line_no++;
 	    if (get_line(fp) < 0) {
 		put_msg("File is truncated at metric/inches or centering specification.");
-		return(-1);
+		return -1;
 	    }
+	    /* read justification spec */
 	    if ((strncasecmp(buf,"center",6) == 0) || 
 		(strncasecmp(buf,"flush",5) == 0)) {
-		center = strncasecmp(buf,"flush",5);
+		/* but set only if user didn't specify it */
+		if (!centerspec)
+		    center = strncasecmp(buf,"flush",5);
 		/* now read metric/inches spec */
 		line_no++;
 		if (get_line(fp) < 0) {
 		    put_msg("File is truncated at metric/inches specification.");
-		    return(-1);
+		    return -1;
 		}
 	    }
+	    /* read metric/inches spec */
 	    /* if metric, scale magnification to correct for xfig display error */
-	    if (strncasecmp(buf,"metric",5)==0)
-		mag *= 80.0/76.2;
+	    if (strncasecmp(buf,"metric",5)==0) {
+		metric = 1;
+	    } else {
+		metric = 0;
+	    }
+	
+	    /* new stuff in 3.2 */
+	    if (v32_flag) {
+		char *p;
+		/* read the paper size */
+		line_no++;
+		if (get_line(fp) < 0) {
+		    put_msg("File is truncated at paper size specification.");
+		    return -1;
+		}
+		if (!paperspec) {
+		    /* copy all except newline */
+		    strncpy(papersize,buf,strlen(buf)-1);
+		    /* and truncate at first blank, if any */
+		    if (p=strchr(papersize,' '))
+			*p = '\0';
+		}
+
+		/* read the magnification */
+		line_no++;
+		if (get_line(fp) < 0) {
+		    put_msg("File is truncated at magnification specification.");
+		    return -1;
+		}
+		/* if the users hasn't specified a magnification on the command line,
+		   use the one in the file */
+		if (!magspec)
+		    mag = atof(buf)/100.0;
+
+		/* read the multiple page flag */
+		line_no++;
+		if (get_line(fp) < 0) {
+		    put_msg("File is truncated at multiple page specification.");
+		    return -1;
+		}
+		if (!multispec)
+		    multi_page = (strncasecmp(buf,"multiple",8) == 0);
+
+		/* read the transparent color.  This is ignored because it is
+		   only used by xfig when exporting to GIF */
+		line_no++;
+		if (get_line(fp) < 0) {
+		    put_msg("File is truncated at transparent color specification.");
+		    return -1;
+		}
+	    } 
+	} else {
+	/* older than 3.1, no metric/inch flag */
+	    metric = 0;
+	}
+
+	/* v 3.1 or older, set paper size unless user asked with -z */
+	if (!v32_flag) {
+	    if (!paperspec) {
+		if (metric)
+		    strcpy(papersize,"A4");
+		else
+		    strcpy(papersize,"Letter");
+	    }
 	}
 
 	line_no++;
 	/* now read for resolution and coord_sys */
 	if (get_line(fp) < 0) {
 	    put_msg("File is truncated at resolution specification.");
-	    return(-1);
+	    return -1;
 	    }
 	if (sscanf(buf,"%d%d\n", &ppi, &coord_sys) != 2) {
 	    put_msg("Incomplete resolution information at line %d", line_no);
-	    return(-1);
+	    return -1;
 	    }
 
 	THICK_SCALE = ppi/80;	/* convert line thickness from screen resolution */
@@ -233,7 +319,7 @@ F_compound	*obj;
 	while (get_line(fp) > 0) {
 	    if (sscanf(buf, "%d", &object) != 1) {
 		put_msg("Incorrect format at line %d", line_no);
-		return(-1);
+		return -1;
 		}
 	    switch (object) {
 		case O_COLOR_DEF:
@@ -247,7 +333,7 @@ F_compound	*obj;
 		    break;
 		case O_POLYLINE :
 		    if ((l = read_lineobject(fp)) == NULL) 
-			return(-1);
+			return -1;
 		    if (ll)
 			ll = (ll->next = l);
 		    else 
@@ -255,8 +341,17 @@ F_compound	*obj;
 		    num_object++;
 		    break;
 		case O_SPLINE :
-		    if ((s = read_splineobject(fp)) == NULL) 
-			return(-1);
+		    if ((s = read_splineobject(fp)) == NULL) { 
+			return -1;
+			}
+		    if (v32_flag){ /* s is a line */
+		      if (ll)
+			ll = (ll->next = (F_line *) s);
+		      else 
+			ll = obj->lines = (F_line *) s;
+		      num_object++;
+		      break;		      
+		    }
 		    if (ls)
 			ls = (ls->next = s);
 		    else 
@@ -265,7 +360,7 @@ F_compound	*obj;
 		    break;
 		case O_ELLIPSE :
 		    if ((e = read_ellipseobject()) == NULL) 
-			return(-1);
+			return -1;
 		    if (le)
 			le = (le->next = e);
 		    else 
@@ -274,7 +369,7 @@ F_compound	*obj;
 		    break;
 		case O_ARC :
 		    if ((a = read_arcobject(fp)) == NULL) 
-			return(-1);
+			return -1;
 		    if (la)
 			la = (la->next = a);
 		    else 
@@ -283,7 +378,7 @@ F_compound	*obj;
 		    break;
 		case O_TEXT :
 		    if ((t = read_textobject(fp)) == NULL) 
-			return(-1);
+			return -1;
 		    if (lt)
 			lt = (lt->next = t);
 		    else 
@@ -292,7 +387,7 @@ F_compound	*obj;
 		    break;
 		case O_COMPOUND :
 		    if ((c = read_compoundobject(fp)) == NULL) 
-			return(-1);
+			return -1;
 		    if (lc)
 			lc = (lc->next = c);
 		    else 
@@ -301,13 +396,13 @@ F_compound	*obj;
 		    break;
 		default :
 		    put_msg("Incorrect object code at line %d", line_no);
-		    return(-1);
+		    return -1;
 		} /*  switch */
 	    } /*  while */
 	if (feof(fp))
-	    return(0);
+	    return 0;
 	else
-	    return(errno);
+	    return errno;
 	} /*  read_objects */
 
 static void
@@ -356,7 +451,7 @@ FILE	*fp;
 
 	if (NULL == (Arc_malloc(a))) {
 	    put_msg(Err_mem);
-	    return(NULL);
+	    return NULL;
 	    }
 	a->pen = 0;
 	a->fill_style = 0;
@@ -388,9 +483,9 @@ FILE	*fp;
 	if ((v30_flag && n != 21) || (!v30_flag && n != 19)) {
 	    put_msg(Err_incomp, "arc", line_no);
 	    free((char*)a);
-	    return(NULL);
+	    return NULL;
 	    }
-	a->thickness *= THICK_SCALE;
+	a->thickness *= round(THICK_SCALE);
 	a->fill_style = FILL_CONVERT(a->fill_style);
 	/* keep track if pattern is used */
 	note_pattern(a->fill_style);
@@ -401,7 +496,7 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "arc", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    a->for_arrow = make_arrow(type, style, thickness, wid, ht);
@@ -412,12 +507,12 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "arc", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    a->back_arrow = make_arrow(type, style, thickness, wid, ht);
 	    }
-	return(a);
+	return a;
 	}
 
 static F_compound *
@@ -445,19 +540,19 @@ FILE	*fp;
 	if (n != 4) {
 	    put_msg(Err_incomp, "compound", line_no);
 	    free((char*)com);
-	    return(NULL);
+	    return NULL;
 	    }
 	while (get_line(fp) > 0) {
 	    if (sscanf(buf, "%d", &object) != 1) {
 		put_msg(Err_incomp, "compound", line_no);
 		free_compound(&com);
-		return(NULL);
+		return NULL;
 		}
 	    switch (object) {
 		case O_POLYLINE :
 		    if ((l = read_lineobject(fp)) == NULL) { 
 			free_line(&l);
-			return(NULL);
+			return NULL;
 			}
 		    if (ll)
 			ll = (ll->next = l);
@@ -467,8 +562,15 @@ FILE	*fp;
 		case O_SPLINE :
 		    if ((s = read_splineobject(fp)) == NULL) { 
 			free_spline(&s);
-			return(NULL);
+			return NULL;
 			}
+		    if (v32_flag){ /* s is a line */
+		      if (ll)
+			ll = (ll->next = (F_line *) s);
+		      else 
+			ll = com->lines = (F_line *) s;
+		      break;		      
+		    }
 		    if (ls)
 			ls = (ls->next = s);
 		    else 
@@ -477,7 +579,7 @@ FILE	*fp;
 		case O_ELLIPSE :
 		    if ((e = read_ellipseobject()) == NULL) { 
 			free_ellipse(&e);
-			return(NULL);
+			return NULL;
 			}
 		    if (le)
 			le = (le->next = e);
@@ -487,7 +589,7 @@ FILE	*fp;
 		case O_ARC :
 		    if ((a = read_arcobject(fp)) == NULL) { 
 			free_arc(&a);
-			return(NULL);
+			return NULL;
 			}
 		    if (la)
 			la = (la->next = a);
@@ -497,7 +599,7 @@ FILE	*fp;
 		case O_TEXT :
 		    if ((t = read_textobject(fp)) == NULL) { 
 			free_text(&t);
-			return(NULL);
+			return NULL;
 			}
 		    if (lt)
 			lt = (lt->next = t);
@@ -507,7 +609,7 @@ FILE	*fp;
 		case O_COMPOUND :
 		    if ((c = read_compoundobject(fp)) == NULL) { 
 			free_compound(&c);
-			return(NULL);
+			return NULL;
 			}
 		    if (lc)
 			lc = (lc->next = c);
@@ -515,16 +617,16 @@ FILE	*fp;
 			lc = com->compounds = c;
 		    break;
 		case O_END_COMPOUND :
-		    return(com);
+		    return com;
 		default :
 		    put_msg("Wrong object code at line %d", line_no);
-		    return(NULL);
+		    return NULL;
 		} /*  switch */
 	    }
 	if (feof(fp))
-	    return(com);
+	    return com;
 	else
-	    return(NULL);
+	    return NULL;
 	}
 
 static F_ellipse *
@@ -560,15 +662,15 @@ read_ellipseobject()
 	if ((v30_flag && n != 19) || (!v30_flag && n != 18)) {
 	    put_msg(Err_incomp, "ellipse", line_no);
 	    free((char*)e);
-	    return(NULL);
+	    return NULL;
 	    }
 	fix_color(&e->pen_color);
 	fix_color(&e->fill_color);
-	e->thickness *= THICK_SCALE;
+	e->thickness *= round(THICK_SCALE);
 	e->fill_style = FILL_CONVERT(e->fill_style);
 	/* keep track if pattern is used */
 	note_pattern(e->fill_style);
-	return(e);
+	return e;
 	}
 
 static F_line *
@@ -628,10 +730,10 @@ FILE	*fp;
 	     (radius_flag && ((!v30_flag && n!=11)||(v30_flag && n!=15)))) {
 	    put_msg(Err_incomp, "line", line_no);
 	    free((char*)l);
-	    return(NULL);
+	    return NULL;
 	    }
-	l->radius *= THICK_SCALE;
-	l->thickness *= THICK_SCALE;
+	l->radius *= round(THICK_SCALE);
+	l->thickness *= round(THICK_SCALE);
 	l->fill_style = FILL_CONVERT(l->fill_style);
 	/* keep track if pattern is used */
 	note_pattern(l->fill_style);
@@ -642,7 +744,7 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "line", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    l->for_arrow = make_arrow(type, style, thickness, wid, ht);
@@ -652,7 +754,7 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "line", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    l->back_arrow = make_arrow(type, style, thickness, wid, ht);
@@ -677,13 +779,13 @@ FILE	*fp;
 
 	if (NULL == (l->points = Point_malloc(p))) {
 	    put_msg(Err_mem);
-	    return(NULL);
+	    return NULL;
 	    }
 	p->next = NULL;
 	if (fscanf(fp, "%d%d", &p->x, &p->y) != 2) {
 	    put_msg(Err_incomp, "line", line_no);
 	    free_linestorage(l);
-	    return(NULL);
+	    return NULL;
 	    }
 	if (!v30_flag)
 		npts = 1000000;
@@ -691,14 +793,14 @@ FILE	*fp;
 	    if (fscanf(fp, "%d%d", &x, &y) != 2) {
 		put_msg(Err_incomp, "line", line_no);
 		free_linestorage(l);
-		return(NULL);
+		return NULL;
 		}
 	    if (!v30_flag && x == 9999) 
 		break;
 	    if (NULL == (Point_malloc(q))) {
 		put_msg(Err_mem);
 		free_linestorage(l);
-		return(NULL);
+		return NULL;
 		}
 	    q->x = x;
 	    q->y = y;
@@ -707,7 +809,7 @@ FILE	*fp;
 	    p = q;
 	    }
 	skip_line(fp);
-	return(l);
+	return l;
 	}
 
 static F_spline *
@@ -715,6 +817,7 @@ read_splineobject(fp)
 FILE	*fp;
 {
 	F_spline	*s;
+	F_line          *l;
 	F_point		*p, *q;
 	F_control	*cp, *cq;
 	int		c, n, x, y, fa, ba;
@@ -747,9 +850,9 @@ FILE	*fp;
 	if ((v30_flag && n != 13) || (!v30_flag && n != 10)) {
 	    put_msg(Err_incomp, "spline", line_no);
 	    free((char*)s);
-	    return(NULL);
+	    return NULL;
 	    }
-	s->thickness *= THICK_SCALE;
+	s->thickness *= round(THICK_SCALE);
 	s->fill_style = FILL_CONVERT(s->fill_style);
 	/* keep track if pattern is used */
 	note_pattern(s->fill_style);
@@ -760,7 +863,7 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "spline", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    s->for_arrow = make_arrow(type, style, thickness, wid, ht);
@@ -770,7 +873,7 @@ FILE	*fp;
 	    line_no++;
 	    if (fscanf(fp, "%d%d%lf%lf%lf", &type, &style, &thickness, &wid, &ht) != 5) {
 		fprintf(stderr, Err_incomp, "spline", line_no);
-		return(NULL);
+		return NULL;
 		}
 	    skip_line(fp);
 	    s->back_arrow = make_arrow(type, style, thickness, wid, ht);
@@ -781,12 +884,12 @@ FILE	*fp;
 	if ((n = fscanf(fp, "%d%d", &x, &y)) != 2) {
 	    put_msg(Err_incomp, "spline", line_no);
 	    free_splinestorage(s);
-	    return(NULL);
+	    return NULL;
 	    };
 	if (NULL == (s->points = Point_malloc(p))) {
 	    put_msg(Err_mem);
 	    free_splinestorage(s);
-	    return(NULL);
+	    return NULL;
 	    }
 	p->x = x; p->y = y;
 	c = 1;
@@ -797,14 +900,14 @@ FILE	*fp;
 		put_msg(Err_incomp, "spline", line_no);
 		p->next = NULL;
 		free_splinestorage(s);
-		return(NULL);
+		return NULL;
 		};
 	    if (!v30_flag && x == 9999) 
 		break;
 	    if (NULL == (Point_malloc(q))) {
 		put_msg(Err_mem);
 		free_splinestorage(s);
-		return(NULL);
+		return NULL;
 		}
 	    q->x = x;
 	    q->y = y;
@@ -815,19 +918,45 @@ FILE	*fp;
 	p->next = NULL;
 	skip_line(fp);
 
-	if (normal_spline(s)) return(s);
+	if (v32_flag)
+	  {
+	    /* transform x-splines into lines */
 
+	    F_control * ptr;
+	    double control_s;
+
+	    make_control_factors(s);
+	    ptr = s->controls;
+	    while (ptr)    /* read controls */
+	      {
+		if ((n = fscanf(fp, "%lf", &control_s)) != 1) {
+		  put_msg(Err_incomp, "spline", line_no);
+		  free_splinestorage(s);
+		  return NULL;
+		}
+		ptr->s = control_s;
+		ptr = ptr->next;
+	      }
+
+	    l = create_line_with_spline(s);
+	    free_splinestorage(s);  
+	    if (l == NULL)
+		return NULL;
+	    return (F_spline *)l;   /* return the new line */
+	  }
+
+	if (approx_spline(s)) return s;
 	skip_comment(fp);
-	/* Read controls */
+	/* Read controls from older versions */
 	if ((n = fscanf(fp, "%lf%lf%lf%lf", &lx, &ly, &rx, &ry)) != 4) {
 	    put_msg(Err_incomp, "spline", line_no);
 	    free_splinestorage(s);
-	    return(NULL);
+	    return NULL;
 	    };
 	if (NULL == (s->controls = Control_malloc(cp))) {
 	    put_msg(Err_mem);
 	    free_splinestorage(s);
-	    return(NULL);
+	    return NULL;
 	    }
 	cp->lx = lx; cp->ly = ly;
 	cp->rx = rx; cp->ry = ry;
@@ -836,13 +965,13 @@ FILE	*fp;
 		put_msg(Err_incomp, "spline", line_no);
 		cp->next = NULL;
 		free_splinestorage(s);
-		return(NULL);
+		return NULL;
 		};
 	    if (NULL == (Control_malloc(cq))) {
 		put_msg(Err_mem);
 		cp->next = NULL;
 		free_splinestorage(s);
-		return(NULL);
+		return NULL;
 		}
 	    cq->lx = lx; cq->ly = ly;
 	    cq->rx = rx; cq->ry = ry;
@@ -852,9 +981,57 @@ FILE	*fp;
 	cp->next = NULL;
 
 	skip_line(fp);
-	return(s);
+	return s;
 	}
 
+/* strncasecmp and strcasecmp by Fred Appelman (Fred.Appelman@cv.ruu.nl) */
+
+#ifdef HAVE_NO_STRNCASECMP
+
+int strncasecmp(const char* s1, const char* s2, int n)
+{
+   char c1,c2;
+
+   while (--n>=0)
+   {
+          /* Check for end of string, if either of the strings
+           * is ended, we can terminate the test
+           */
+          if (*s1=='\0' && *s2!='\0') return -1; /* s1 ended premature */
+          if (*s1!='\0' && *s2=='\0') return +1; /* s2 ended premature */
+
+          c1=toupper(*s1++);
+          c2=toupper(*s2++);
+          if (c1<c2) return -1; /* s1 is "smaller" */
+          if (c1>c2) return +1; /* s2 is "smaller" */
+   }
+   return 0;
+}
+
+#endif
+
+#ifdef HAVE_NO_STRCASECMP
+int strcasecmp(const char* s1, const char* s2)
+{
+   char c1,c2;
+
+   while (*s1 && *s2)
+   {
+          c1=toupper(*s1++);
+          c2=toupper(*s2++);
+          if (c1<c2) return -1; /* s1 is "smaller" */
+          if (c1>c2) return +1; /* s2 is "smaller" */
+   }
+   /* Check for end of string, if not both the strings ended they are
+    * not the same.
+        */
+   if (*s1=='\0' && *s2!='\0') return -1; /* s1 ended premature */
+   if (*s1!='\0' && *s2=='\0') return +1; /* s2 ended premature */
+   return 0;
+}
+
+#endif
+ 
 static F_text *
 read_textobject(fp)
 FILE	*fp;
@@ -939,7 +1116,7 @@ FILE	*fp;
 	if ((n != 14) && (n != 13)) {
 	  put_msg(Err_incomp, "text", line_no);
 	  free((char*)t);
- 	  return(NULL);
+ 	  return NULL;
 	}
 #endif
 	if (font_size) {
@@ -968,7 +1145,6 @@ FILE	*fp;
 	if (more) {
 	/* Read in the subsequent lines of the text if there is more than one. */
 	  do {
-	    fgets(buf, BUF_SIZE, fp);
 	    line_no++;		/* As is done in get_line */
 	    if (v30_flag) {
 		if (fgets(s_temp, BUF_SIZE, fp) == NULL)
@@ -987,7 +1163,7 @@ FILE	*fp;
 		n = sscanf(buf, "%[^\1]%[\1]", s_temp, junk);
 	    }
 	    /* Safety check */
-	    if (strlen(s)+1 + strlen(s_temp)+1 > BUF_SIZE) {
+	    if (strlen(s)+1 + strlen(s_temp)+1 > (size_t)BUF_SIZE) {
 	      /* Too many characters.  Ignore the rest. */
 	      ignore = 1;
 	    }
@@ -1004,7 +1180,7 @@ FILE	*fp;
 			if (l < len && s[l+1] != '\\') {
 			    if (sscanf(&s[l+1],"%3o",&num)!=1) {
 				put_msg("Error in parsing text string on line",line_no);
-				return(NULL);
+				return NULL;
 			    }
 			    buf[n++]= (unsigned char) num;	/* put char in */
 			    l += 3;			/* skip over digits */
@@ -1025,7 +1201,7 @@ FILE	*fp;
 	if (NULL == t->cstring) {
 	    put_msg(Err_mem);
 	    free((char*)t);
-	    return(NULL);
+	    return NULL;
 	}
 	(void)strcpy(t->cstring, s+1);
 
@@ -1041,7 +1217,7 @@ FILE	*fp;
 	if (t->font > MAXFONT(t))
 		t->font = MAXFONT(t);
 	fix_color(&t->color);
-	return(t);
+	return t;
 }
 
 /* akm 28/2/95 - count consecutive backslashes backwards */
@@ -1066,18 +1242,18 @@ FILE	*fp;
 {
 	while (1) {
 	    if (NULL == fgets(buf, BUF_SIZE, fp)) {
-		return(-1);
+		return -1;
 		}
 	    line_no++;
 	    if (*buf != '\n' && *buf != '#') 
-		return(1);	/* Skip empty and comment lines */
+		return 1;	/* Skip empty and comment lines */
 	    }
 }
 
 skip_comment(fp)
 FILE	*fp;
 {
-	char c;
+	int c;
 
 	while ((c = fgetc(fp)) == '#') 
 		skip_line(fp);
